package SPVM::Go::Poll;



1;

=head1 Name

SPVM::Go::Poll - IO Polling

=head1 Description

The Go::Poll class in L<SPVM> has methods to poll read/write IO.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

