package SPVM::Go::Context::CanncelCallback;



1;

=encoding utf8

=head1 Name

SPVM::Go::Context::CanncelCallback - Short Description

=head1 Description

Go::Context::CanncelCallback interface in L<SPVM> has interface methods to do someting.

=head1 Usage

  interface Go::Context::CanncelCallback;

=head1 Details



=head1 Interface Methods




=head1 Repository



=head1 Author

Yuki Kimoto C<kimoto.yuki@gmail.com>

=head1 Copyright & License

Copyright (c) 2026 Yuki Kimoto

MIT License

