package SPVM::Go::Schedule::Task;



1;

=head1 Name

SPVM::Go::Schedule::Task - Shceduling Task

=head1 Description

The Go::Schedule::Task class in L<SPVM> has a task to schedule goroutnies.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

