package SPVM::Go::Schedule;



1;

=head1 Name

SPVM::Go::Schedule - Scheduling Goroutines

=head1 Description

The Go::Schedule class in L<SPVM> has methods to schedule goroutines.

=head1 Usage

  use Go::Schedule;

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

